
from lxml import etree as ET


class DMARCRuaParser:
    def __init__(self, dns):
        self.dns = dns

    def parse(self, rua_report):
        root = ET.parse(rua_report)
        records = root.xpath(
            (
                ".//record["
                "./row/policy_evaluated/disposition != 'none' or "
                "./row/policy_evaluated/spf != 'pass' or "
                "./row/policy_evaluated/dkim != 'pass' or "
                "./auth_results/dkim/result != 'pass' or "
                "./auth_results/spf/result!='pass'"
                "]"
            )
        )
        data = []
        for record in records:
            row = record[0]
            source_ip = row[0].text

            source_host = self.dns.reverse_name(source_ip)

            dmarc_policy_evalution = row[2]
            dmarc_disposition = dmarc_policy_evalution[0].text
            dkim_align = dmarc_policy_evalution[1].text
            spf_align = dmarc_policy_evalution[2].text

            auth_results = record[2]
            dkim_auth = next(
                filter(lambda result: result != 'pass',
                       map(lambda result: result.text, auth_results.xpath("./dkim/result"))
                       ), 'pass')
            spf_auth = next(
                filter(lambda result: result != 'pass',
                       map(lambda result: result.text, auth_results.xpath("./spf/result"))
                       ), 'pass')

            identifiers = record[1]
            payload_from = identifiers[0].text
            envelop_from = next(
                filter(lambda result: result,
                       map(lambda result: result.text,
                           auth_results.xpath("./spf/domain"))
                       ), '')

            data.append([
                source_ip, source_host,
                payload_from, envelop_from,
                dmarc_disposition,
                dkim_align, dkim_auth, spf_align, spf_auth
            ])
        return data



import gzip
import zipfile


class DecompressorFactory():
    def create(file_path):
        if file_path.endswith(".gz"):
            return GZipDecompressor()
        if file_path.endswith(".zip"):
            return ZipDecompressor()
        raise NotImplementedError(file_path)


class GZipDecompressor():
    def decompress(self, gzip_file_path):
        with gzip.open(gzip_file_path, 'rb') as f:
            return f.read()


class ZipDecompressor():
    def decompress(self, zip_file_path):
        with zipfile.ZipFile(zip_file_path, 'r') as zf:
            files = zf.infolist()
            with zf.open(files[0].filename) as f:
                return f.read()




import socket


class DNS:
    def reverse_name(self, ipv4):
        result = ''
        try:
            result = socket.gethostbyaddr(ipv4)[0]
        except socket.herror as e:
            result = e.strerror
        return result




from os import listdir
from os.path import isfile
from os.path import join
import io

from DMARCReporting.decompressor import DecompressorFactory
from DMARCReporting.parser import DMARCRuaParser
from DMARCReporting.renderer import ConsoleRenderer
from DMARCReporting.dns import DNS


class CLI():
    def execute(self, input_dir):
        files = sorted([
            f for f in listdir(input_dir) if isfile(join(input_dir, f))
        ])

        parser = DMARCRuaParser(DNS())
        renderer = ConsoleRenderer()

        for file in files:
            file_path = join(input_dir, file)
            report = DecompressorFactory.create(file_path).decompress(file_path)
            data = parser.parse(io.BytesIO(report))
            renderer.render(file, data)




from tabulate import tabulate


class ConsoleRenderer:
    def render(self, file, data):
        if len(data) == 0:
            return

        print()
        print(file)
        print(tabulate(data, headers=[
            "Source IP",
            "Source Host",
            "Payload From (From:)",
            "Envelop From (MAIL FROM)",
            "DMARC",
            "DKIM Align",
            "DKIM Auth",
            "SPF Align",
            "SPF Auth"
        ]))



import pathlib
from setuptools import setup, find_packages

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="DMARCReporting",
    version="0.1.1",
    description="Simple tool to extract error reports from DMARC files",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/MozaicWorks/DMARCReporting",
    author="Mozaic Works",
    author_email="alex.bolboaca@mozaicworks.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3"
    ],
    packages=find_packages(exclude=("tests",)),
    install_requires=["lxml", "tabulate"],
    python_requires='>=3.8',
    entry_points={
        'console_scripts': [
            'DMARCReporting=DMARCReporting:main',
        ],
    },
)
